#ifndef ___PRIMO_PLATFORM_CONFIG_H___
#define ___PRIMO_PLATFORM_CONFIG_H___

/** @file */

// Microsoft Visual C++ Versions:
// Microsoft Visual C++ 7.1	_MSC_VER = 1310
// Microsoft Visual C++ 7.0	_MSC_VER = 1300
// Microsoft Visual C++ 6.0	_MSC_VER = 1200

#if defined(_WIN32) || defined(_WIN64)
    // Windows
    #if !defined(PLATFORM_WINDOWS)
        #define PLATFORM_WINDOWS
    #endif		
#elif defined(__APPLE__)
    // Mac OS X
    #if !defined(PLATFORM_MACOSX)
        #define PLATFORM_MACOSX
    #endif		
#elif defined (__linux__)
    // Linux
    #if !defined(PLATFORM_LINUX)
        #define PLATFORM_LINUX
    #endif		
#endif

#if defined(__POWERPC__)
    #if !defined(PLATFORM_BIG_ENDIAN)
        #define PLATFORM_BIG_ENDIAN
    #endif	
#else
    #if !defined(PLATFORM_LITTLE_ENDIAN)
        #define PLATFORM_LITTLE_ENDIAN
    #endif	
#endif

// Common headers
#if defined(PLATFORM_WINDOWS)
    #include <windows.h>

    // throw is not supported by VC 6.0
    // Microsoft Visual C++ 2002, 2003, 2005 _MSC_VER > 1200  
    #if _MSC_VER > 1200
            #define ___PRIMO_THROW		throw(...)		
            #define ___PRIMO_NO_THROW	throw()
    #else
    // Microsoft Visual C++ 6.0	_MSC_VER = 1200
            #define ___PRIMO_THROW
            #define ___PRIMO_NO_THROW
    #endif

    #define ___PLATFORM_HAS_BOOL
#elif defined(PLATFORM_MACOSX)
    //#include <TargetConditionals.h>
    //#include <Carbon/Carbon.h>
    #include <CoreFoundation/CoreFoundation.h>

    #define ___PRIMO_THROW
    #define ___PRIMO_NO_THROW
#elif defined(PLATFORM_LINUX)
    #define ___PRIMO_THROW
    #define ___PRIMO_NO_THROW
#else
    #error "Unknown platform."
#endif

// Integer Types
#if defined(PLATFORM_WINDOWS)
    #if !defined(___PRIMO_INT_T)
        // Microsoft Visual C++ 2010 _MSC_VER >= 1600
        #if _MSC_VER >= 1600
            // Microsoft Visual C++ 2010 has the int types
            #include <stdint.h>
        // Microsoft Visual C++ 2002 2003, 2005 _MSC_VER >= 1300 
        #elif _MSC_VER >= 1300
            typedef unsigned __int8  uint8_t;
            typedef unsigned __int16 uint16_t;
            typedef unsigned __int64 uint64_t;

            typedef	__int8	 int8_t;
            typedef	__int16  int16_t;
            typedef __int64  int64_t;

            typedef unsigned int uint32_t;
            typedef int int32_t;

            typedef	uint8_t	 byte;
        #else
            // Microsoft Visual C++ 6.0	_MSC_VER = 1200
            #define uint8_t		BYTE
            #define uint16_t	WORD
            #define uint64_t	unsigned __int64 

            #define int8_t		char
            #define int16_t		short
            #define int64_t		__int64

            #define uint32_t DWORD
            #define int32_t LONG
        #endif

        #define ___PRIMO_INT_T
    #endif
#elif defined(PLATFORM_MACOSX)
    // Unix has the int types already
	#include <stdint.h>
    #include <time.h>
#elif defined(PLATFORM_LINUX)
    // Unix has the int types already
    #include <stdint.h>
    #include <time.h>
#else
	#error "Unknown platform."
#endif

// bool_t 
#if !defined(___PRIMO_BOOL_T)
    #if defined(___PLATFORM_HAS_BOOL)
        typedef BOOL bool_t;
    #else
        typedef int bool_t;
    #endif

    #define ___PRIMO_BOOL_T
#endif

// char_t
#if !defined(___PRIMO_CHAR_T)
    #if defined(PLATFORM_WINDOWS)
        // utf16 character
        typedef wchar_t utf16_t;

        // utf8 character
        typedef uint8_t utf8_t;

        // char_t 
        #if defined(PRIMO_UTF8)
                typedef utf8_t char_t;
        #else
                typedef wchar_t char_t;
        #endif

        #define ___PRIMO_CHAR_T
    #elif defined(PLATFORM_MACOSX) 
        // utf16 character
        typedef uint16_t utf16_t;

        // utf8 character
        typedef uint8_t utf8_t;

        // char_t
        #if defined(PRIMO_UTF8)
            typedef utf8_t char_t;
        #else
            typedef utf16_t char_t;
        #endif
        
        #define ___PRIMO_CHAR_T
    #else
        // utf16 character
        typedef uint16_t utf16_t;

        // utf8 character
        typedef uint8_t utf8_t;

        // char_t
        #if defined(PRIMO_UTF8)
            typedef utf8_t char_t;
        #else
            typedef utf16_t char_t;
        #endif

        #define ___PRIMO_CHAR_T            
    #endif
#endif

// tchar_t
#if !defined(___PRIMO_TCHAR_T)
    typedef wchar_t tchar_t;
    #define ___PRIMO_TCHAR_T
#endif

// handle_t 
#if !defined(___PRIMO_HANDLE_T)
    #if defined(PLATFORM_WINDOWS)
        typedef HANDLE handle_t;
        #define ___PRIMO_HANDLE_T
    #elif defined(PLATFORM_MACOSX) || defined(PLATFORM_LINUX)
        typedef void* handle_t;
        #define ___PRIMO_HANDLE_T
    #else
        #error "Unknown platform."
    #endif
#endif

// filetime_t
#if !defined(___PRIMO_FILETIME_T)
    #if defined(PLATFORM_WINDOWS)
        typedef FILETIME filetime_t;
        #define ___PRIMO_FILETIME_T
    #elif defined(PLATFORM_MACOSX) || defined(PLATFORM_LINUX)
        typedef time_t filetime_t;
        #define ___PRIMO_FILETIME_T
    #else
        #error "Unknown platform."
    #endif
#endif

//  Workaround for the unfortunate min/max macros defined by some platform headers
#if defined PRIMO_NO_MIN_MAX
    #if defined(max)
        #undef max
    #endif

    #if defined(min)
        #undef min
    #endif
#endif

// Define Deprecated Attribute
#if !defined(PRIMO_DEPRECATED)
    #if _MSC_VER >= 1400 //VC 2005 and later
        #define PRIMO_DEPRECATED(Text) __declspec(deprecated(Text))
    #elif defined(_MSC_VER)
        #define PRIMO_DEPRECATED(Text) __declspec(deprecated)
    #elif defined(__clang__) || defined(__INTEL_COMPILER) || ((__GNUC__ * 100 + __GNUC_MINOR__) > 404)
        #define PRIMO_DEPRECATED(Text) __attribute__ ((deprecated(Text)))
    #else
        #define PRIMO_DEPRECATED(Text) __attribute__ ((deprecated))
    #endif
#endif


#endif // ___PRIMO_PLATFORM_CONFIG_H___

